unit LbBack;

interface

uses
   {$IFDEF WIN32} Windows, {$ELSE} WinTypes, WinProcs, {$ENDIF} Graphics, Classes, Forms;

type
   TLbBackground = class(TComponent)
   private
      FBitmap: TBitmap;
      FOwner: TForm;
      FEnabled: boolean;
      FOldPaint: TNotifyEvent;
   public
      constructor Create(aOwner: TComponent); override;
      destructor Destroy; override;
      procedure Loaded; override;
   protected
      procedure SetBitmap(aNew: TBitmap);
      procedure SetEnabled(bNew: boolean);

      procedure DoPaint(Sender: TObject);
   published
      property Enabled: boolean read FEnabled write SetEnabled;
      property Image: TBitmap read FBitmap write SetBitmap;
   end;

procedure Register;

implementation

{------------------------------------------------------------------------------}

procedure Register;
begin
   RegisterComponents('Samples', [TLbBackground]);
end;

{##############################################################################}

constructor TLbBackground.Create(aOwner: TComponent);
begin
   inherited;

   FOwner := TForm(aOwner);
   FBitmap := TBitmap.Create;
   FOldPaint := nil;
end;

{------------------------------------------------------------------------------}

destructor TLbBackground.Destroy;
begin
   FBitmap.Free;
   FOwner.OnPaint := FOldPaint;

   inherited;
end;

{------------------------------------------------------------------------------}

procedure TLbBackground.Loaded;
begin
   if assigned(Fowner.OnPaint) then FOldPaint := FOwner.OnPaint;
   FOwner.OnPaint := DoPaint;
end;

{------------------------------------------------------------------------------}

procedure TLbBackground.SetBitmap(aNew: TBitmap);
begin
   FBitmap.Assign(aNew);
   FOwner.Invalidate;
end;

{------------------------------------------------------------------------------}

procedure TLbBackground.SetEnabled(bNew: boolean);
begin
   FEnabled := bNew;
   FOwner.Invalidate;
end;

{------------------------------------------------------------------------------}

procedure TLbBackground.DoPaint(Sender: TObject);
var
   X, Y: integer;
   aRect, bRect: TRect;
   iWidth, iHeight: integer;

begin
   if FEnabled and not (FBitmap.Empty) then
   begin
      iWidth := FBitmap.Width;
      iHeight := FBitmap.Height;

      bRect.Left := 0;
      bRect.Top := 0;
      bRect.Right := iWidth;
      bRect.Bottom := iHeight;

      aRect.Left := 0;
      aRect.Top := 0;
      aRect.Right := iWidth;
      aRect.Bottom := iHeight;

      for X := 0 to (FOwner.Width div iWidth) + 1 do
      begin
         for Y := 0 to (FOwner.Height div iHeight) + 1 do
         begin
            FOwner.Canvas.CopyRect(aRect, FBitmap.Canvas, bRect);
            aRect.Top := aRect.Bottom;
            aRect.Bottom := aRect.Bottom + iHeight;
         end;
         aRect.Left := aRect.Right;
         aRect.Right := aRect.Right + iWidth;
         aRect.Top := 0;
         aRect.Bottom := iHeight;
      end;
   end;

   if assigned(FOldPaint) then FOldPaint(Sender);
end;

end.

